@@ -2,8 +2,8 @@
 #
 # Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
 #     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
-#     2005, 2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute
-#     of Technology
+#     2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014
+#     Massachusetts Institute of Technology
 #
 # This file is part of MIT/GNU Scheme.
 #
@@ -69,7 +69,12 @@ if [ ${FULL} = yes ]; then
 fi
 
 if [ ${DIST} = yes ]; then
-    maybe_rm Makefile boot-lib config.cache config.log config.status
+    maybe_rm Makefile
+    maybe_rm Makefile.tools
+    maybe_rm boot-lib           # XXX What's this?
+    maybe_rm config.cache
+    maybe_rm config.log
+    maybe_rm config.status
 fi
 
 if [ ${MAINTAINER} = yes ]; then
