@@ -2,8 +2,8 @@
 #
 # Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
 #     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
-#     2005, 2006, 2007, 2008, 2009, 2010 Massachusetts Institute of
-#     Technology
+#     2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014
+#     Massachusetts Institute of Technology
 #
 # This file is part of MIT/GNU Scheme.
 #
@@ -25,11 +25,11 @@
 # Utility to set up the MIT/GNU Scheme compiler directory.
 # The working directory must be the compiler directory.
 
-set -e
+set -eu
 
 . ../etc/functions.sh
 
-../etc/Setup.sh "${@}"
+../etc/Setup.sh ${@:+"${@}"}
 
 for N in 1 2 3; do
     maybe_link machines/vax/dinstr${N}.scm instr${N}.scm
