@@ -2,8 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
-    Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Massachusetts
+    Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -155,26 +155,19 @@ USA.
 
 ;;;; Noops
 
-(package (cfg-node-tag/noop! cfg-node-tag/noop?)
+(define-integrable (cfg-node-tag/noop! tag)
+  (set-vector-tag-noop! tag true))
 
-(define-export (cfg-node-tag/noop! tag)
-  (vector-tag-put! tag noop-tag-property true))
-
-(define-export (cfg-node-tag/noop? tag)
-  (vector-tag-get tag noop-tag-property))
-
-(define noop-tag-property
-  "noop-tag-property")
-
-)
+(define-integrable (cfg-node-tag/noop? tag)
+  (vector-tag-noop tag))
 
 (define-integrable (cfg-node/noop? node)
   (cfg-node-tag/noop? (tagged-vector/tag node)))
 
 (define noop-node-tag
-  (make-vector-tag snode-tag 'NOOP false))
-
-(cfg-node-tag/noop! noop-node-tag)
+  (let ((tag (make-vector-tag snode-tag 'NOOP false)))
+    (cfg-node-tag/noop! tag)
+    tag))
 
 (define-integrable (make-noop-node)
   (let ((node (make-snode noop-node-tag)))
