@@ -2,8 +2,8 @@
 #
 # Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
 #     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
-#     2005, 2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute
-#     of Technology
+#     2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014
+#     Massachusetts Institute of Technology
 #
 # This file is part of MIT/GNU Scheme.
 #
@@ -56,16 +56,15 @@ export MIT_SCHEME_EXE
 script_name="${0}"
 report_error ()
 {
-  local line
   cat <<EOF
 *** Error in ${script_name}
 
 This script builds MIT/GNU Scheme a native back end, starting from
 MIT/GNU Scheme with the portable C back end.
 EOF
-  printf '\n'
+  echo
   cat
-  printf '\n'
+  echo
   cat <<EOF
 If you have installed MIT/GNU Scheme with the portable C back end in
 an unusual location, set the environment variable MIT_SCHEME_EXE to
