@@ -1,10 +1,9 @@
 /* -*-C-*-
 
-$Id: uxfs.c,v 1.32 2008/01/30 20:02:22 cph Exp $
-
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Massachusetts
+    Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -124,31 +123,35 @@ USA.
 #  define FILE_TOUCH_OPEN_TRIES 5
 #endif
 
+#define STAT_SYSTEM_CALL(name, expression, if_failure)	\
+  do {							\
+    while ((expression) < 0)				\
+      {							\
+	if ((errno == ENOENT) || (errno == ENOTDIR))	\
+	  if_failure;					\
+	if (errno != EINTR)				\
+	  error_system_call (errno, name);		\
+	deliver_pending_interrupts ();			\
+      }							\
+  } while (0)
+
+void
+UX_read_fd_status (int fd, struct stat * s)
+{
+  STD_VOID_SYSTEM_CALL (syscall_fstat, (UX_fstat (fd, s)));
+}
+
 int
 UX_read_file_status (const char * filename, struct stat * s)
 {
-  while ((UX_lstat (filename, s)) < 0)
-    {
-      if (errno == EINTR)
-	continue;
-      if ((errno == ENOENT) || (errno == ENOTDIR))
-	return (0);
-      error_system_call (errno, syscall_lstat);
-    }
+  STAT_SYSTEM_CALL (syscall_lstat, (UX_lstat (filename, s)), return (0));
   return (1);
 }
 
 int
 UX_read_file_status_indirect (const char * filename, struct stat * s)
 {
-  while ((UX_stat (filename, s)) < 0)
-    {
-      if (errno == EINTR)
-	continue;
-      if ((errno == ENOENT) || (errno == ENOTDIR))
-	return (0);
-      error_system_call (errno, syscall_stat);
-    }
+  STAT_SYSTEM_CALL (syscall_stat, (UX_stat (filename, s)), return (0));
   return (1);
 }
 
@@ -241,13 +244,7 @@ UX_file_system_type (const char * name)
 {
 #ifdef HAVE_STATFS
   struct statfs s;
-  while ((UX_statfs (name, (&s))) < 0)
-    {
-      if ((errno == ENOENT) || (errno == ENOTDIR))
-	return (0);
-      if (errno != EINTR)
-	error_system_call (errno, syscall_statfs);
-    }
+  STAT_SYSTEM_CALL (syscall_statfs, (UX_statfs (name, (&s))), return (0));
 
 #ifdef __linux__
   switch (s . f_type)
@@ -482,9 +479,10 @@ OS_file_touch (const char * filename)
 	transaction_commit ();
 #else
 	transaction_commit ();
+	/* FIXME: Need to check for EINTR.  */
 	fd = (UX_open (filename, (O_WRONLY | O_TRUNC), MODE_REG));
 	if (fd >= 0)
-	  STD_VOID_SYSTEM_CALL (syscall_close, (UX_close (fd)));
+	  (void) UX_close (fd);
 #endif
 	return (0);
       }
@@ -507,7 +505,7 @@ OS_file_touch (const char * filename)
 static void
 protect_fd_close (void * ap)
 {
-  UX_close (* ((int *) ap));
+  (void) UX_close (* ((int *) ap));
 }
 
 static void
@@ -612,9 +610,11 @@ OS_directory_valid_p (unsigned int index)
 unsigned int
 OS_directory_open (const char * name)
 {
-  DIR * pointer = (opendir (name));
-  if (pointer == 0)
-    error_system_call (errno, syscall_opendir);
+  DIR * pointer;
+  STD_PTR_SYSTEM_CALL (syscall_opendir, pointer, (opendir (name)));
+  /* FIXME: This leaks a directory pointer if malloc or realloc fails
+     here.  Avoiding this safely with a transaction is unfortunately
+     not straightforward.  */
   return (allocate_directory_pointer (pointer));
 }
 
