@@ -2,8 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
-    Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Massachusetts
+    Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -530,6 +530,8 @@ struct full_sigcontext
 
 #ifdef __linux__
 
+#  include <ucontext.h>
+
 /* This isn't really the right test: what we really want to know is if
    the kernel supports the newer signal-delivery mechanism.  */
 
@@ -608,7 +610,7 @@ typedef struct
    minus 1 long.  */
 
 #      define DECLARE_SIGCONTEXT(scp, arg)				\
-  SIGCONTEXT_T * scp;							\
+  SIGCONTEXT_T * scp ATTRIBUTE((unused));				\
   scp = ((SIGCONTEXT_T *) (((unsigned long *) (& (arg))) - 1))
 
 #    endif /* __IA32__ */
@@ -760,7 +762,7 @@ typedef struct
 
 #ifndef DECLARE_SIGCONTEXT
 #  define DECLARE_SIGCONTEXT(scp, arg)					\
-     SIGCONTEXT_T * scp;						\
+     SIGCONTEXT_T * scp ATTRIBUTE ((unused));				\
      scp = ((SIGCONTEXT_T *) (arg))
 #endif
 
