@@ -2,8 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
-    Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Massachusetts
+    Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -130,4 +130,10 @@ USA.
 (define (force* value)
   (cond ((procedure? value) (force* (value)))
 	((promise? value) (force* (force value)))
-	(else value)))
\ No newline at end of file
+	(else value)))
+
+(define (standard-system-loader name)
+  (let ((here (merge-pathnames (pathname-as-directory name)
+			       (directory-pathname (current-load-pathname)))))
+    (lambda ()
+      (with-working-directory-pathname here (lambda () (load "make"))))))
\ No newline at end of file
