@@ -2,8 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
-    Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Massachusetts
+    Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -621,6 +621,22 @@ these rules:
 	      (else #f)))
       (%find-library-directory)))
 
+(define (with-system-library-directories directories thunk)
+
+  (define (existing-directory directory)
+    (let ((dirpath (pathname-as-directory (merge-pathnames directory))))
+      (if (file-directory? dirpath)
+	  dirpath
+	  (error:file-operation dirpath
+				"find" "directory" "no such directory"
+				'with-system-library-directories
+				directories))))
+
+  (fluid-let ((library-directory-path
+	       (append (map existing-directory directories)
+		       library-directory-path)))
+    (thunk)))
+
 (define (%find-library-directory)
   (pathname-simplify
    (or (find-matching-item library-directory-path file-directory?)
